@@ -265,10 +265,14 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       netfilter_derived_probe *np = probes[i];
       s.op->newline() << "static unsigned int enter_netfilter_probe_" << np->nf_index;
 
-      // Previous to kernel 2.6.22, the hookfunction definition takes a struct sk_buff **skb,
-      // whereas currently it uses a *skb. We need emit the right version so this will
-      // compile on RHEL5, for example.
-      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V313";
+      // Previous to kernel 2.6.22, the hookfunction definition takes
+      // a struct sk_buff **skb, whereas currently it uses a *skb. We
+      // need emit the right version so this will compile on RHEL5,
+      // for example.
+      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V41";
+      s.op->newline() << "(const struct nf_hook_ops *nf_ops, struct sk_buff *nf_skb, const struct nf_hook_state *nf_state)";
+      s.op->newline() << "{";
+      s.op->newline() << "#elif defined(STAPCONF_NETFILTER_V313)";
 
       s.op->newline() << "(const struct nf_hook_ops *nf_ops, struct sk_buff *nf_skb, const struct net_device *nf_in, const struct net_device *nf_out, int (*nf_okfn)(struct sk_buff *))";
       s.op->newline() << "{";
@@ -287,9 +291,14 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->newline(-1) << "#endif";
       s.op->newline(1) << "const struct stap_probe * const stp = & stap_probes[" << np->session_index << "];";
       s.op->newline() << "int nf_verdict = NF_ACCEPT;"; // default NF_ACCEPT, to be used by $verdict context var
-      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V313";
+      s.op->newline() << "#if defined(STAPCONF_NETFILTER_V313) || defined(STAPCONF_NETFILTER_V41)";
       s.op->newline() << "unsigned int nf_hooknum = nf_ops->hooknum;";
       s.op->newline() << "#endif";
+      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V41";
+      s.op->newline() << "struct net_device *nf_in = nf_state->in;";
+      s.op->newline() << "struct net_device *nf_out = nf_state->out;";
+      s.op->newline() << "int (*nf_okfn)(struct sock *, struct sk_buff *) = nf_state->okfn;";
+      s.op->newline() << "#endif";
       common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp",
                                      "stp_probe_type_netfilter",
                                      false);
