@@ -13,31 +13,28 @@
 #define GRND_RANDOM 0x0002
 #endif
 
-#if defined(__NR_getrandom) && !defined(SYS_getrandom)
-#define SYS_getrandom __NR_getrandom
-#endif
-
 int main()
 {
-#ifdef SYS_getrandom
+// e.g. kernel-headers-3.13.9-200.fc20.x86_64 do not have __NR_getrandom
+#ifdef __NR_getrandom
   char i[5] = {0};
   char j[5] = {0};
   char k[5] = {0};
 
-  syscall(SYS_getrandom, i, sizeof(i), GRND_RANDOM);
+  syscall(__NR_getrandom, i, sizeof(i), GRND_RANDOM);
   //staptest// getrandom ("", 5, GRND_RANDOM) = 5
 
-  syscall(SYS_getrandom, (size_t)-1, 5, GRND_RANDOM);
+  syscall(__NR_getrandom, (size_t)-1, 5, GRND_RANDOM);
 #if __WORDSIZE == 64
   //staptest// getrandom ([16]?[f]+, 5, GRND_RANDOM) = -NNNN (EFAULT)
 #else
   //staptest// getrandom ([8]?[f]+, 5, GRND_RANDOM) = -NNNN (EFAULT)
 #endif
 
-  syscall(SYS_getrandom, j, -1, GRND_RANDOM);
+  syscall(__NR_getrandom, j, -1, GRND_RANDOM);
   //staptest// getrandom ("", 4294967295, GRND_RANDOM) = NNNN
 
-  syscall(SYS_getrandom, k, sizeof(k), -1);
+  syscall(__NR_getrandom, k, sizeof(k), -1);
   //staptest// getrandom ("", 5, GRND_[^ ]+|XXXX) = -NNNN (EINVAL)
 #endif
   return 0;
